//#include <stdlib.h> 
//#include <stdint.h>
//#include <stdio.h>
//#include <string.h>
#include <avr/io.h>
//#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/eeprom.h>
#include <math.h>

#include "uart.h"
#include "i2c.h"
#include "dpy.h"
#include "gps.h"
#include "icon.h"
#include "compass.h"



#define COMPASS_I2C_ADDR 0x3c  // 0x78?

#define COMPASS_REG_CONFIGURATION_REGISTER_A	00
#define COMPASS_REG_CONFIGURATION_REGISTER_B	01
#define COMPASS_REG_MODE_REGISTER				02
#define COMPASS_REG_DATA_OUTPUT_X_MSB_REGISTER	03
#define COMPASS_REG_DATA_OUTPUT_X_LSB_REGISTER	04
#define COMPASS_REG_DATA_OUTPUT_Z_MSB_REGISTER	05
#define COMPASS_REG_DATA_OUTPUT_Z_LSB_REGISTER	06
#define COMPASS_REG_DATA_OUTPUT_Y_MSB_REGISTER	07
#define COMPASS_REG_DATA_OUTPUT_Y_LSB_REGISTER	08
#define COMPASS_REG_STATUS_REGISTER				09
#define COMPASS_REG_IDENTIFICATION_REGISTER_A	10
#define COMPASS_REG_IDENTIFICATION_REGISTER_B	11
#define COMPASS_REG_IDENTIFICATION_REGISTER_C	12

// Compass Calibration
#define COMPASS_ADJ_X_AVG						-34;
#define COMPASS_ADJ_X_SPAN						867;
#define COMPASS_ADJ_Y_AVG						-216;
#define COMPASS_ADJ_Y_SPAN						884;

// I obtained the x_min, x_max, y_min, y_max values through testing with my device.
// These values are different on another digital compass device, please take note and adjust for your device.
// You can test your max/avg/span values by moving around your digital compass, hence calibrating.


/*
Min en max values can be calculated:
  avg = (min+max)/2 
  span = max - min 

Example table for my device:

		min  max      avg  span
  -----------------|-----------
  X  	-411 +467  |   28   878
  Y  	-615 +252  | -182   868
  Z  	-331 +430  |   59   661

The axis not only all have a different offset, but also a different scale.
Normalizing these to a range e.g. -1 .. +1, for each axis you would get:

  value = (value - avg) / (span / 2)

The 3 values are now corrected for offset and scale, you can now do atan2 and get the right angle.

*/

struct compass_data {
	int16_t x;
	int16_t z; // Do read Z as well in one go, even we if don't use it, z is complex to use.
	int16_t y;
};


struct compass_adjust {
	double x_min;
	double x_max;
	double y_min;
	double y_max;
	double x_avg;
	double x_span;
	double y_avg;
	double y_span;
};

volatile struct compass_adjust ca = {0};

void compass_write_reg(uint8_t regno, uint8_t v)
{
	i2c_start();
	i2c_write(COMPASS_I2C_ADDR | I2C_WRITE);
	i2c_write(regno);
	i2c_write(v);
	i2c_stop();
}


void compass_read_regs(uint8_t regno, void *buf, size_t count)
{
	i2c_start();
	i2c_write(COMPASS_I2C_ADDR | I2C_WRITE);
	i2c_write(regno);
	i2c_restart();
	i2c_write(COMPASS_I2C_ADDR | I2C_READ);

	while(count) {
		if(count > 1) {
			i2c_read(buf);
		} else {
			i2c_read_last(buf);
		}
		count --;
		buf ++;
	}
	i2c_stop();
}


void compass_init(void)
{
	compass_write_reg(COMPASS_REG_MODE_REGISTER, 0x00);
}


double compass_poll(void)
{
	struct compass_data cd;
	ca.x_avg = COMPASS_ADJ_X_AVG;
	ca.x_span = COMPASS_ADJ_X_SPAN;
	ca.y_avg = COMPASS_ADJ_Y_AVG;
	ca.y_span = COMPASS_ADJ_Y_SPAN;
	compass_read_regs( COMPASS_REG_DATA_OUTPUT_X_MSB_REGISTER, &cd, sizeof cd );
	cd.x = ((cd.x & 0x00ff) << 8) | ((cd.x & 0xff00) >> 8); //compose most significant and least significant byte
	cd.y = ((cd.y & 0x00ff) << 8) | ((cd.y & 0xff00) >> 8); //compose most significant and least significant byte
	double x = ( cd.x - ca.x_avg ) / ( ca.x_span / 2 );
	double y = ( cd.y - ca.y_avg ) / ( ca.y_span / 2 );
	double a = atan2( y, x );
	return a;
}


/*
 * End
 */

