#include "evq.h"


struct evhandler {
	enum ev_type mask;
	void (*fn)(ev *e, void *data);
	void *data;
};


#define EVQ_SIZE 16
#define EVH_SIZE 16


struct evq {
	ev evs[EVQ_SIZE];
	struct evhandler ehs[EVH_SIZE];
	uint8_t head;
	uint8_t tail;
};


volatile struct evq evq;


void evq_register(enum ev_type mask, void (*fn)(ev *e, void *data), void *data)
{
	int i;
	for(i=0; i<EVH_SIZE; i++) {
		if(evq.ehs[i].mask == 0) {
			evq.ehs[i].mask = mask;
			evq.ehs[i].fn = fn;
			evq.ehs[i].data = data;
			return;
		}
	}
}


void evq_init(void)
{
	evq.head = 0;
	evq.tail = 0;
}


void evq_push(ev *ev)
{
	evq.evs[evq.head] = *ev;
	evq.head = (evq.head + 1) % EVQ_SIZE;
}


void evq_pop(ev *ev)
{
	if(evq.head == evq.tail) return;

	*ev = evq.evs[evq.tail];
	evq.tail = (evq.tail +1) % EVQ_SIZE;

	int i;
	for(i=0; i<EVH_SIZE; i++) {
		if(evq.ehs[i].mask & ev->type) {
			evq.ehs[i].fn(ev, evq.ehs[i].data);
		}
	}
}


void evq_wait(ev *ev)
{
	while(evq.head == evq.tail);
	evq_pop(ev);
}

/*
 * End
 */