#include <stdlib.h> 
#include <stdio.h>
#include <string.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/eeprom.h>
#include <math.h>


#include "init.h"					// Definitions for this project for Atmel Atmega1284
#include "util.h"					// Additional support functions
#include "uart.h"					// UART functions
#include "i2c.h"					// I2C functions
#include "dpy.h"					// Display functions
#include "gps.h"					// GPS functions
#include "icon.h"					// Icon data
#include "compass.h"				// Compass functions
#include "vibrate.h"				// Vibrate functions
#include "settings.h"				// Settings functions - to do
#include "evq.h"					// Event queue functions - to do
#include "sd.h"						// SD Card hardware implementation - to do
#include "fat32.h"					// FAT32 implementation - to do


int main(void)
{
	// Startup
	init_spi();		// set MCU's SPI, see init.h & init.c
	uart_init();	// Initialise UART, see uart.h & uart.c
	dpy_init();		// Initialise Display, see dpy.h & dpy.c
	sd_init();		// Initialise SD Card (settings), see sd.h & sd.c
	sei();			// Enable Global Interrupts, see datasheet
	
	// Power light on
	PWR_LED_REG |= (1<<PWR_LED_PIN);			// Init register, see init.h
	PWR_LED_PORT |= (1<<PWR_LED_PIN);			// OR debug led at PWR_LED_PIN, on.
	
	printf("\r\n");								// empty row
	printf("\r\n");								// empty row
	printf("\r\n");								// empty row
	
	// Indication startup
	printf("#TGPS1.0 * Startup.\r\n");
	// Show CPU 
	printf("#TGPS1.0 * Atmel %s on %lu\r\n", F_MCU, F_CPU);
	
	//printf("#TGPS1.0 * Atmel F_MCU on %lu\r\n", F_CPU);

	// Intro
	printf("#TGPS1.0 * Intro.\r\n");	
	dpy_intro();
	vibrate();
	
	// Text test 
	// printf("#TGPS1.0 * Text test.\r\n");	
	// dpy_clear_screen();
	// dpy_printf(0,  0, "%02d-%02d-%02d    ", 1, 2, 3);
	// dpy_flush();
	delay_ms(1000);
	
	// Main loop
	printf("#TGPS1.0 * Main.\r\n");
	
	// main loop
	// struct gps gps;
	uint8_t result;
	
	while(1) {
		// Show something like it is working
		PWR_LED_PORT ^= (1<<PWR_LED_PIN);		// XOR debug led at PA3	off is on, on is off and again.

		//dpy_clear();
		//draw_icon();						// See draw_icon function, flush afterwards
		
		result = sd_init_card();
		dpy_printf(0, 0, "%d", result);
		dpy_flush();
		
		PWR_LED_PORT ^= (1<<PWR_LED_PIN);		// XOR debug led at PA3	off is on, on is off and again.
		//dpy_flush();
	}
}

		
/*
 * End
 */

 
/*
 * vim: ts=3 sw=3 ft=c
 */