
#include <stdint.h>

#include "init.h"
#include "sd.h"
#include "util.h"

// temporary define
#define NOT_OK 	0

/*
from init.h: SD CARD (SDC)

#define SDC_CTRL_REG		DDRC				// DDRA
#define SDC_PORT  			PORTC				// PORTA
#define SDC_CS_PIN 			PC2					// PC2

http://en.wikipedia.org/wiki/Transistor
transistor: the base switches current between collector and emitter.
So I could use a transistor on a pin, to actively turn the card off/on when wanted.

*/

/*
Command
Index	Argument	Response	Data	Abbreviation	Description
CMD0	None(0)	R1	No	GO_IDLE_STATE	Software reset.
CMD1	None(0)	R1	No	SEND_OP_COND	Initiate initialization process.
ACMD41(*1)	*2	R1	No	APP_SEND_OP_COND	For only SDC. Initiate initialization process.
CMD8	*3	R7	No	SEND_IF_COND	For only SDC V2. Check voltage range.
CMD9	None(0)	R1	Yes	SEND_CSD	Read CSD register.
CMD10	None(0)	R1	Yes	SEND_CID	Read CID register.
CMD12	None(0)	R1b	No	STOP_TRANSMISSION	Stop to read data.
CMD16	Block
length[31:0]	R1	No	SET_BLOCKLEN	Change R/W block size.
CMD17	Address[31:0]	R1	Yes	READ_SINGLE_BLOCK	Read a block.
CMD18	Address[31:0]	R1	Yes	READ_MULTIPLE_BLOCK	Read multiple blocks.
CMD23	Number of
blocks[15:0]	R1	No	SET_BLOCK_COUNT	For only MMC. Define number of blocks to transfer
with next multi-block read/write command.
ACMD23(*1)	Number of
blocks[22:0]	R1	No	SET_WR_BLOCK_ERASE_COUNT	For only SDC. Define number of blocks to pre-erase
with next multi-block write command.
CMD24	Address[31:0]	R1	Yes	WRITE_BLOCK	Write a block.
CMD25	Address[31:0]	R1	Yes	WRITE_MULTIPLE_BLOCK	Write multiple blocks.
CMD55(*1)	None(0)	R1	No	APP_CMD	Leading command of ACMD<n> command.
CMD58	None(0)	R3	No	READ_OCR	Read OCR.
*/

/*
#define GO_IDLE_STATE            0
#define SEND_OP_COND             1
#define SEND_IF_COND			 8
#define SEND_CSD                 9
#define STOP_TRANSMISSION        12
#define SEND_STATUS              13
#define SET_BLOCK_LEN            16
#define READ_SINGLE_BLOCK        17
#define READ_MULTIPLE_BLOCKS     18
#define WRITE_SINGLE_BLOCK       24
#define WRITE_MULTIPLE_BLOCKS    25
#define ERASE_BLOCK_START_ADDR   32
#define ERASE_BLOCK_END_ADDR     33
#define ERASE_SELECTED_BLOCKS    38
#define SD_SEND_OP_COND			 41   //ACMD
#define APP_CMD					 55
#define READ_OCR				 58
#define CRC_ON_OFF               59

*/

// Set CS high
void sd_cs_assert(void)
{
	//
}

// Set CS low
void sd_cs_deassert(void)
{
	//
}

// Initialise all settings for sdcards
void sd_init(void)
{
	// Set Chip Select (CS), currently that is the SDC_CS_PIN on SDC_PORT, we'll use that as CS
	SDC_PORT |= (1<<SDC_CS_PIN);
}


// Initialise sdcard, if inserted
uint8_t sd_init_card(void)
{
	uint8_t count=0;
	// At power on, wait more than 1 ms
	delay_ms(2);
	
	// Let's use CS, CS should be high/low when clock cycling.
	
	// Do at least 80 dummy clocks
	// Send 0xff. that is 11111111, more than 8 times.
	// We do this while CS is high, and make DI high with sending 0xff's
	
	while(count<11) { 
		// transmit 0xff through SPI
		count++;
	}
	
	// Send CMD0 CRC must be correct for this command
	//if !(sd_send_cmd(CMD0)) return 1;
	
	
	
	
	// Set SPI in slow mode
		// Let's not do that and assume we're slow enough at the moment.
	
	
	
	if(NOT_OK) { return 1; };
	return 0; 
	
	// return codes
	// 0 is OK
	// 1 is some error
	// 2 is SD Ver 1
	// 3 is SD Ver 2 byte addressing (force block size of 512b)
	// 4 is SD Ver 2 block addressing
	
}


// Send command to sd card
uint8_t sd_send_cmd(char c)
{
	// how what why
	
	// if CMD == CMD0 then CRC must be set
	return 0;
}

// Reset a card, if errors on an inserted card occur
uint8_t sd_reset(void)
{
	//
	return 0;
}


// Flush memory buffer to sd card
uint8_t sd_flush(void)
{
	// write block in memory
	return 0;
}


// Power down an inserted card, saving power, needs init to reestablish comms
uint8_t sd_powerdown(void)
{
	// I have no transistor in place yet, but may want to switch on/off the module
	return 0;
}



// Power up an inserted card, using power, calls init as well to reestablish comms
uint8_t sd_powerup(void)
{
	// I have no transistor in place yet, but may want to switch on/off the module
	return 0;
}


// Write a block
uint8_t sd_write_block(void)
{
	//
	return 0;
}


// Reads a block
uint8_t sd_read_block(void)
{
	//
	return 0;
}


uint8_t sd_cmd(void)
{
	//
	return 0;
}

/*
 * End
 */
